@@ -18,6 +18,8 @@ pub struct Args {
     pub url: String,
     pub qtype: DnsType,
     pub short: bool,
+    #[cfg(feature = "json")]
+    pub json: bool,
     pub use_dnssec: bool,
     pub connection_type: ConnectionType,
     pub port: u16,
@@ -35,6 +37,8 @@ pub fn parse_args() -> Args {
     let mut url = "example.com".to_string();
     let mut qtype = DnsType::A;
     let mut short = false;
+    #[cfg(feature = "json")]
+    let mut json = false;
     let mut use_dnssec = false;
     let mut connection_type = ConnectionType::UDP;
     let mut port = None;
@@ -60,6 +64,10 @@ pub fn parse_args() -> Args {
                 "short" => {
                     short = true;
                 }
+                #[cfg(feature = "json")]
+                "json" => {
+                    json = true;
+                }
                 "do" => {
                     use_dnssec = true;
                 }
@@ -148,6 +156,8 @@ pub fn parse_args() -> Args {
         url,
         qtype,
         short,
+        #[cfg(feature = "json")]
+        json,
         use_dnssec,
         connection_type,
         port: port.unwrap_or(53),
@@ -162,7 +172,9 @@ fn print_help() {
     println!("       \t    -h | --help            (print this help message)");
     println!("       \t    -x                     (shortcut for reverse lookup)");
     println!("       \tflags is one or more of the following:");
-    println!("       \t    +short  (only print answer section of response)");
+    println!("       \t    +short  (only print answer sections of response)");
+    #[cfg(feature = "json")]
+    println!("       \t    +json   (format output as JSON)");
     println!("       \t    +do     (fetch DNSSEC records)");
     println!("       \t    +tcp    (use TCP instead of UDP)");
     #[cfg(feature = "tls")]
