@@ -2,6 +2,8 @@ use byteorder::{NetworkEndian, ReadBytesExt, WriteBytesExt};
 use chrono::{TimeZone, Utc};
 use data_encoding::{BASE32HEX, BASE64};
 use rand::Rng;
+#[cfg(feature = "json")]
+use serde::Serialize;
 use std::fmt::{self, Display};
 use std::io::{Cursor, Read, Seek, SeekFrom};
 use std::{cmp::max, collections::HashMap};
@@ -14,6 +16,7 @@ use error::{EncodeError, ParseError};
 /// Represents a DNS OpCode.
 /// See [here](https://www.iana.org/assignments/dns-parameters/dns-parameters.xhtml#dns-parameters-5) for
 /// further information.
+#[cfg_attr(feature = "json", derive(Serialize))]
 #[derive(PartialEq, Copy, Clone, Debug)]
 pub enum DnsOpcode {
     QUERY,
@@ -27,6 +30,7 @@ pub enum DnsOpcode {
 /// Represents a DNS RCODE, including those introduced by EDNS.
 /// See [here](https://www.iana.org/assignments/dns-parameters/dns-parameters.xhtml#dns-parameters-6) for
 /// further information.
+#[cfg_attr(feature = "json", derive(Serialize))]
 #[derive(PartialEq, Copy, Clone, Debug)]
 pub enum DnsRcode {
     NOERROR,
@@ -54,6 +58,7 @@ pub enum DnsRcode {
 /// Represents a DNS TYPE.
 /// This enum is non-exhaustive, see [here](https://en.wikipedia.org/wiki/List_of_DNS_record_types) for a
 /// more comprehensive overview as well as explanations and links to the respective defining RFCs.
+#[cfg_attr(feature = "json", derive(Serialize))]
 #[derive(PartialEq, Copy, Clone, EnumString, Debug)]
 pub enum DnsType {
     A,
@@ -98,6 +103,7 @@ pub enum DnsType {
 /// Represents a DNS CLASS.
 /// See [RFC 1035](https://tools.ietf.org/pdf/rfc1035.pdf) for further information.
 /// Other classes than IN and ANY are included only for completeness and historical reasons.
+#[cfg_attr(feature = "json", derive(Serialize))]
 #[derive(PartialEq, Copy, Clone, Debug)]
 pub enum DnsClass {
     IN,
@@ -108,6 +114,7 @@ pub enum DnsClass {
 }
 
 /// Represents the flags of a `DnsHeader`.
+#[cfg_attr(feature = "json", derive(Serialize))]
 pub struct DnsFlags {
     pub flags: u16,
 }
@@ -117,6 +124,7 @@ pub struct DnsFlags {
 /// See [here](https://www.iana.org/assignments/dns-parameters/dns-parameters.xhtml#dns-parameters-12) for
 /// a list of available flags and their respective defining RFCs. The general format of a header is
 /// defined in [RFC 1035](https://tools.ietf.org/pdf/rfc1035.pdf).
+#[cfg_attr(feature = "json", derive(Serialize))]
 pub struct DnsHeader {
     pub msg_id: u16, // supplied by questioner and reflected back unchanged by responder
     pub qr: bool,    // false for query, true for response
@@ -131,6 +139,7 @@ pub struct DnsHeader {
 
 /// Represents a DNS question, i.e. an entry in the question section of a DNS message.
 /// See [RFC 1035](https://tools.ietf.org/pdf/rfc1035.pdf) for further information.
+#[cfg_attr(feature = "json", derive(Serialize))]
 pub struct DnsQuestion {
     pub qname: String,
     pub qtype: DnsType,
@@ -140,6 +149,7 @@ pub struct DnsQuestion {
 /// Represents a DNS record, i. e. an entry in the answer, authority or additional section of a DNS
 /// message.
 /// See [RFC 1035](https://tools.ietf.org/pdf/rfc1035.pdf) for further information.
+#[cfg_attr(feature = "json", derive(Serialize))]
 #[derive(Clone)]
 pub enum DnsRecord {
     OPT {
@@ -152,7 +162,9 @@ pub enum DnsRecord {
         edns_version: u8,
         flags: HashMap<&'static str, bool>,
         // rdlength omitted as rdata knows its own length
-        rdata: Vec<u8>,            // needed for encoding
+        #[cfg_attr(feature = "json", serde(skip))]
+        rdata: Vec<u8>, // needed for encoding
+        #[cfg_attr(feature = "json", serde(rename = "rdata"))]
         parsed_rdata: Vec<String>, // see DnsRecord::parse_rdata() for more information
     },
     NONOPT {
@@ -161,13 +173,16 @@ pub enum DnsRecord {
         class: DnsClass,
         ttl: u32,
         // rdlength omitted as rdata knows its own length
-        rdata: Vec<u8>,            // needed for encoding
+        #[cfg_attr(feature = "json", serde(skip))]
+        rdata: Vec<u8>, // needed for encoding
+        #[cfg_attr(feature = "json", serde(rename = "rdata"))]
         parsed_rdata: Vec<String>, // see DnsRecord::parse_rdata() for more information
     },
 }
 
 /// Represents a DNS message.
 /// See [RFC 1035](https://tools.ietf.org/pdf/rfc1035.pdf) for further information.
+#[cfg_attr(feature = "json", derive(Serialize))]
 pub struct DnsMessage {
     pub header: DnsHeader,
     pub questions: Vec<DnsQuestion>,
