@@ -15,9 +15,8 @@ use args::ConnectionType;
 // TODO
 // - see if we can get nicer error messages
 // - add +trace support
-// - JSON output format
 // - DoH
-// - Colors for output
+// - Colors for output (highlight answer section)
 // - add tests for parsing (look at cargo fuzz)
 // - DNSSEC verification?
 
@@ -245,6 +244,11 @@ fn send_query_tls(args: &args::Args, data: &[u8]) -> Result<(Vec<u8>, u16, Durat
 
 fn display_result(res: &DnsMessage, args: &args::Args, bytes_recvd: u16, elapsed: &Duration) {
     if !args.short {
+        #[cfg(feature = "json")]
+        if args.json {
+            println!("{}", serde_json::to_string_pretty(&res).unwrap());
+            return;
+        }
         println!("{}", res);
         println!();
         println!("Query metadata:");
@@ -254,14 +258,32 @@ fn display_result(res: &DnsMessage, args: &args::Args, bytes_recvd: u16, elapsed
         return;
     }
 
+    let all_answers: Vec<_> = res
+        .answers
+        .iter()
+        .chain(res.authoritative_answers.iter())
+        .chain(
+            res.additional_answers
+                .iter()
+                // OPT records don't have an `as_padded_string()` implementation
+                .filter(|record| matches!(record, DnsRecord::NONOPT { .. })),
+        )
+        .collect();
+
+    #[cfg(feature = "json")]
+    if args.json {
+        println!("{}", serde_json::to_string_pretty(&all_answers).unwrap());
+        return;
+    }
+
     let (mut max_owner_len, mut max_type_len) = (0, 0);
-    for answer in &res.answers {
+    for answer in &all_answers {
         if let DnsRecord::NONOPT { name, atype, .. } = answer {
             max_owner_len = max(max_owner_len, name.len());
             max_type_len = max(max_type_len, atype.to_string().len());
         }
     }
-    for answer in &res.answers {
+    for answer in &all_answers {
         println!("{}", answer.as_padded_string(max_owner_len, max_type_len));
     }
 }
